#include "testApp.h"


//--------------------------------------------------------------
void testApp::setup(){

    catMew.loadSound("mew.mp3");


	ofSetVerticalSync(true);
	ofSetFrameRate(60);

    xPos = ofRandom(50,ofGetWidth()-150);
    yPos = ofRandom(5,200);
    penWidth    = 300;
    penHeight   = 250;

    rat.loadImage("ratty.png");
    ratPosX = ofRandom(50,ofGetWidth()-50);
    ratPosY = ofRandom(ofGetHeight()-100,ofGetHeight()-200);

    inPen = false;
    numPennedCats = 0;

	for (int i = 0; i < 8; i++){
		particle myParticle;
		myParticle.setInitialCondition(ofRandom(200,1000),ofRandom(100,1000),0,0);
		myParticle.setColor(ofRandom(0,250),ofRandom(0,250),ofRandom(0,250));
		// more interesting with diversity :)
		// uncomment this:
		myParticle.damping = ofRandom(0.01, 0.05);
		particles.push_back(myParticle);


	}

	bRepel		= true;
	radius		= 40;
	strength	= 0.25f;


}

//--------------------------------------------------------------
void testApp::update(){

	// on every frame
	// we reset the forces
	// add in any forces on the particle
	// perfom damping and
	// then update



	for (int i = 0; i < particles.size(); i++){
		particles[i].resetForce();
		particles[i].addRepulsionForce(mouseX, mouseY, 100, .5);
		if (!inPen){
	     particles[i].addAttractionForce (ratPosX, ratPosY, 2000, .005);
      }
	}


	// zach's way !!!!
	for (int i = 0; i < particles.size(); i++){
		for (int j = 0; j < i; j++){
			if (bRepel){
				particles[i].addRepulsionForce(particles[j], 100, .3);
			} else {
				particles[i].addAttractionForce(particles[j], 50, .003);
			}
		}
	}

	for (int i = 0; i < particles.size(); i++){
		particles[i].addDampingForce();
		particles[i].update();
	}

	for (int i = 0; i < particles.size(); i++){
        if(particles[i].inPen){
            catMew.play();
            }
	}

}

//--------------------------------------------------------------
void testApp::draw(){
    //draw Pen
    drawPen(xPos,yPos, penWidth, penHeight);
    //draw Rat
    drawRat(ratPosX,ratPosY);
    testWin();

	ofEnableAlphaBlending();
	ofSetColor(0,130,130, 200);

	ofSetColor(0x000000);

	for (int i = 0; i < particles.size(); i++){
		particles[i].draw();
	}




}

//--------------------------------------------------------------

void testApp::drawPen (float xPos, float yPos, float penWidth, float penHeight){
    ofSetColor(150,80,50);
    ofRect(xPos,yPos, penWidth, penHeight);

        //(particles[i].pos.x + (particles[i].imgW)/2)
        //(particles[i].pos.y + (particles[i].imgH)/2)


  for (int i = 0; i < particles.size(); i++){
    if(particles[i].pos.x > xPos && particles[i].pos.x < xPos+penWidth && particles[i].pos.y > yPos && particles[i].pos.y < yPos+penHeight){
    if((particles[i].pos.x + (particles[i].imgW)/2) > xPos && (particles[i].pos.x + (particles[i].imgW)/2) < xPos+penWidth && (particles[i].pos.y + (particles[i].imgH)/2) > yPos && (particles[i].pos.y + (particles[i].imgH)/2) < yPos+penHeight){

      particles[i].inPen = true;
      numPennedCats +=1;

    }
    }
    else {
      particles[i].inPen = false;
    }
   //cout<<"i: "<< i << " in pen: "<< particles[i].inPen;

}

}
void testApp::drawRat(float ratPosX,float ratPosY){

  ofSetColor(150,150,150);

  //ofEllipse(ratPosX,ratPosY,20,20);
  rat.draw(ratPosX,ratPosY,50,30);

}

void testApp::testWin(){

    numPennedCats = 0;

    for (int i = 0; i < particles.size(); i++){

        if (particles[i].inPen){
            numPennedCats = numPennedCats + 1;
            }


        printf("%i \n", numPennedCats);

        int remainingCats = particles.size() - numPennedCats;
        string myString = "You have " + ofToString(remainingCats) + " left to herd!" ;

    //"(space) = reset\n(a/s) strength = " + ofToString(strength) +
    //                        "\n(z/x) radius = " + ofToString(radius) +
    //                        "\n(r) toggle mode = " + (bRepel ? "repelling" : "attracting");

        ofSetColor(40,40,0);
        ofRect(5,5,220,50);
        ofSetColor(255,255,255);
        ofDrawBitmapString(myString, 30, 30);
    }



}


//--------------------------------------------------------------
void testApp::keyPressed  (int key){

	switch (key){

		case ' ':
			// reposition everything:
			for (int i = 0; i < particles.size(); i++){
				particles[i].setInitialCondition(ofRandom(0,ofGetWidth()),ofRandom(0,ofGetHeight()),0,0);
			}
			break;
		case 'a':
			strength += 0.02f;
			break;
		case 's':
			strength -= 0.02f;
			if (strength <= 0) strength = 0.02f;
			break;
		case 'z':
			radius += 5.0f;
			break;
		case 'x':
			radius -= 5.0f;
			if (radius <= 0) radius = 5.0f;
			break;
		case 'r':
			bRepel = !bRepel;
			break;
	}


}

//--------------------------------------------------------------
void testApp::keyReleased  (int key){
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	particles.erase(particles.begin());
	particle myParticle;
	myParticle.setInitialCondition(x,y,0,0);
	particles.push_back(myParticle);
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mouseReleased(){
}
